///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Jul 19, 2006	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.ccl;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.ccl.TdbCclCommand;
import com.tietoenator.trip.jxp.ccl.TdbHistoryDetail;
import com.tietoenator.trip.jxp.ccl.TdbKernelWindow;
import com.tietoenator.trip.jxp.ccl.TdbKernelWindowType;
import com.tietoenator.trip.jxp.ccl.TdbTerm;
import com.tietoenator.trip.jxp.ccl.TdbTermList;
import com.tietoenator.trip.jxp.ccl.TdbTermTree;
import com.tietoenator.trip.jxp.ccl.TdbTreeEntry;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbInterfaceType;
import com.tietoenator.trip.jxp.session.TdbNotificationType;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * A very simple TTY-mode CCL interpreter, working very much like TRIPclassic
 * in TTY-mode. Specify CCL commands at the '*' prompt and see them executed
 * interactively.
 */
public class TRIPtty
{
   private TdbCclCommand m_command;
   private TdbSession m_session;
   private ConsoleComforter m_comforter;

   /**
    * Constructor
    * @throws TdbException
    */
   public TRIPtty()
           throws TdbException
   {
      // Create a session on a physical server (no grid support)
      m_session = SessionFactory.newInstance("**** Example CCL Interpreter Using TdbCclCommand ****", false);
   }

   public void setup(boolean useComforter)
           throws TdbException
   {
      // Show what we are connected to.
      System.out.println();
      System.out.println("Connected to TRIP " + m_session.getVersion() + " using the " +
              m_session.getCharacterSet() + " [" + m_session.getCharacterSetName() + "] character set.");
      System.out.println();

      // Create the CCL command interpreter
      m_command = new TdbCclCommand(m_session);

      // Map in a term list to hold Display command results
      m_command.setTermList(new TdbTermList());

      // Map in a term tree to hold Display Down()/Up()/etc. command results
      m_command.setTermTree(new TdbTermTree());

      // Map in a kernel window buffer for each type of output
      m_command.mapKernelWindow(new TdbKernelWindow(m_session, TdbKernelWindowType.Show));
      m_command.mapKernelWindow(new TdbKernelWindow(m_session, TdbKernelWindowType.Expand));
      m_command.mapKernelWindow(new TdbKernelWindow(m_session, TdbKernelWindowType.SysInfo));

      if (useComforter)
      {
          // Install a notification sink for search comforter messages
          m_comforter = new ConsoleComforter(m_session);

          try
          {
            m_session.setNotificationSink(m_comforter);
            m_session.enableNotification(TdbNotificationType.COMFORTER, 4);
          }
          catch (TdbException x)
          {
              System.out.println("WARNING: Unable to install a comforter.");
              System.out.println("  CAUSE: "+x.getMessage());
              System.out.println();
          }
      }
   }
   
   // Utility function used to recurse through a tree of thesaurus terms
   private static final String spaces = "                                                      ";

   private void printThesTerm(TdbTreeEntry entry, int indent)
   {
      System.out.println(spaces.substring(0, indent) + entry.getName());
      for (TdbTreeEntry child : entry.children())
      {
         printThesTerm(child, indent + 2);
      }
   }

   /**
    * Execute a CCL command, post-process it to show the results on the screen
    * @param cmd The command to execute
    * @return true if the user did something signalling a desire to stop using CCL
    */
   public boolean ExecCcl(String cmd)
   {
      TdbKernelWindow w;
      String dym;
      boolean finished = false;
      int i;

      // We get a null command for an empty line.
      if (null == cmd)
         return false;

      try
      {
         // Execute the CCL command typed in by the user
         if (m_comforter != null) {
            m_comforter.reset();
         }
         m_command.resetMessages();
         m_command.execDirect(cmd);
         for (String msg : m_command.messageList())
         {
            System.out.println(msg);
         }

         // Post-process the command
         switch (m_command.getCommandType())
         {

         // Commands that seek to abort the CCL session (STOP)
         case Abort:
            m_session.logout();
            finished = true;
            break;

         // Commands that update the search history in some way (BASE, FIND, FUZZ, DEFINE, LIST, etc.)
         case HistoryReplace:
         case HistoryUpdate:
            for (TdbHistoryDetail detail : m_command.historyUpdates())
            {
               System.out.println("Search " + detail.getSearchId() + ": " +
                       detail.getRecordCount() + " records in database " +
                       m_command.getCurrentBase());
            }
            dym = m_command.getDidYouMean();
            if (dym.length()>0)
               System.out.println("Did you mean: " + dym);
            break;

         // Commands that produce lists of terms (DISPLAY)
         case TermList:
         {
            TdbTermList list = m_command.getTermList();
            i = 1;
            System.out.println("" + list.size() + " terms");
            for (TdbTerm term : list)
            {
               System.out.println("T=" + i++ + "\t<" + term.getRecordCount() + ">\t" + term.getTerm());
            }
            break;
         }

         // Commands that produce trees of terms (Display UP/DOWN/etc.)
         case TermTree:
         {
            TdbTermTree tree = m_command.getTermTree();
            System.out.println("" + tree.getRootCount() + " main terms.\n");
            for (TdbTreeEntry entry : tree.roots())
            {
               printThesTerm(entry, 0);
            }
            break;
         }

         // Commands that produce output (SHOW, MORE, BACK, NEXT, PREV, etc.)
         case Output:
            w = m_command.getAffectedWindow();
//				System.out.println(w.toFormattedString(">>", "<<", "\n"));
            System.out.println(w.toString());
            if (!w.isAtBottom())
            {
               System.out.println("Type More to continue");
            }
            break;
         }
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
      }

      return finished;
   }

   public TdbSession getSession() {
       return m_session;
   }

   /**
    * Entry point for the program
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         TRIPtty tty = new TRIPtty();
         Console stdin = new Console();
         boolean done = false;

         int usecmf = 0;
         if (tty.getSession().getSessionType() == TdbInterfaceType.NET) {
             usecmf = stdin.readint("Install a search comforter (0-No, 1-Yes)", 0, true);
         }

         tty.setup(usecmf!=0);

         while (!done)
         {
            done = tty.ExecCcl(stdin.readln("* ", null, false));
         }
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
      }
   }
}
